/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_ALG_TEST_TRAVERSE_VISITOR__
#define __DD_ALG_TEST_TRAVERSE_VISITOR__

#include <deque>
#include <vector>

#include "DDTreeNode.h"
#include "DDSingleSetTraverseVisitor.h"
#include "DDRankCalculator.h"
#include "DDSingleSetRay.h"

namespace double_description {

    class AlgTestTraverseVisitor : public SingleSetTraverseVisitor {

        private:
        const Matrix & mSystem;
        RankCalculator mCalculator;

        public:
        AlgTestTraverseVisitor( const TreeNode & tree,
                                const unsigned short plane,
                                const Matrix & system,
                                RayCache<SingleSetRay, BitPatternTreeCacheEntry<SingleSetRay> > & cache ) :
            SingleSetTraverseVisitor( tree, system.getColumns() - 2, plane, system.getRows(), cache ),
            mSystem( system ),
            mCalculator( system.getRows(), system.getColumns() ) {}

        void onVisit ( const struct RayReference & current ) {
            
            if ( current.activeSet.getPopulationCount() == mMinCommonBits + 1 ) {
                SingleSetTraverseVisitor::onVisit( current );
            } else {
                RayRefVisitor::onVisit( current );
                mTree.preliminaryTraverse( current, mCandidates );   

                for ( std::deque<struct EdgeCandidate>::const_iterator it = mCandidates.begin(); it != mCandidates.end(); it++ ) {
                    mCalculator.load ( mSystem, it->activeSet );      

                    if ( mCalculator.hasMinimumRank ( mMinCommonBits ) ) {
                        create( *it );
                    }
                }    

                mCandidates.clear();         
            }
        }

        ~AlgTestTraverseVisitor () {}

    };
}

#endif // __DD_ALG_TEST_TRAVERSE_VISITOR__
