/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "DDBitList.h"

#include <sstream>
#include <iostream>

using namespace double_description;

const BitCountTable BitList::BIT_COUNT_TABLE;

void BitList::init ( const unsigned short blocks, const unsigned long value ) {

#ifdef __DOUBLE_BLOCK_OPT__
	mFirstBlock = value;
	mSecondBlock = value;

	data = new unsigned long[blocks - 2];

	for ( unsigned short block = 0; block < blocks-2; block++ ) {
		data[block] = value;
	}

#else
	#ifdef __SINGLE_BLOCK_OPT__

	mFirstBlock = value;

	data = new unsigned long[blocks - 1];

	for ( unsigned short block = 0; block < blocks-1; block++ ) {
		data[block] = value;
	}

	#else

	delete data;
	data = new unsigned long[blocks];

	for ( unsigned short block = 0; block < blocks; block++ ) {
		data[block] = value;
	}

	#endif // __SINGLE_BLOCK_OPT__
#endif //__DOUBLE_BLOCK_OPT__
}

void BitList::resize ( const unsigned long blocks ) {

	unsigned long* copy = NULL;

#ifdef __DOUBLE_BLOCK_OPT__

    if ( __builtin_expect( blocks > 2, false )) {

        copy = new unsigned long[ blocks - 2 ];
        
        if ( blocks > mBlocks ) {
            memset ( copy + ( mBlocks - 2 ), 0, ( blocks - mBlocks ) * sizeof( unsigned long ) );

            if ( __builtin_expect( mBlocks > 2, false )) {
		        memcpy ( copy, data, ( mBlocks - 2 ) * sizeof( unsigned long ) );
		        delete[] data;
            }
        } else {
		    memcpy ( copy, data, ( blocks - 2 ) * sizeof( unsigned long ) );
		    delete[] data;            
        }
    }

	if ( __builtin_expect ( mBlocks > 2, false ) ) {  

		copy = new unsigned long[ mBlocks - 2 ];
		memcpy ( copy, data, ( mBlocks - 2 ) * sizeof( unsigned long ) );
		delete[] data;
	}

    if ( __builtin_expect ( blocks > 2, false ) ) {
	    data = new unsigned long[blocks - 2];
        memcpy ( data, copy, ( blocks - 2 ) * sizeof( unsigned long ) );	
    }

	if ( copy ) {
		delete[] copy;
	}

#else
	#ifdef __SINGLE_BLOCK_OPT__

	copy = new unsigned long[ blocks - 1 ];

    if ( likely( blocks > mBlocks ) ) {
        memset ( copy, 0, ( blocks - 1 ) * sizeof( unsigned long ) );
        memcpy ( copy, data, ( mBlocks - 1 ) * sizeof( unsigned long ) );
    } else {
        memcpy ( copy, data, ( blocks - 1 ) * sizeof( unsigned long ) );
    }

	delete[] data;
	data = copy;

	#else

	copy = new unsigned long[ blocks ];

    if ( likely( blocks > mBlocks ) ) {
        memset ( copy, 0, blocks * sizeof( unsigned long ) );
        memcpy ( copy, data, mBlocks * sizeof( unsigned long ) );
    } else {
        memcpy ( copy, data, blocks * sizeof( unsigned long ) );
    }

	delete[] data;
	data = copy;

	#endif // __SINGLE_BLOCK_OPT__
#endif //__DOUBLE_BLOCK_OPT__

	mBlocks = blocks;
}

const std::string BitList::toString () const {
	return toString( getCapacity() );
}

const std::string BitList::toString ( const unsigned short length ) const {
	std::stringstream output;	

	for ( unsigned long pos = 0; pos < length; pos++ ) {
	    if ( isSet( pos ) ) {
		    output << "1";
		} else {
			output << "0";
		}
	}

	return output.str();
}
