/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_BIT_SELECTOR__
#define __DD_BIT_SELECTOR__

#include <stack>
#include <vector>

#include "DDBitList.h"
#include "DDBitSelectStrategy.h"

namespace double_description {

    class BitSelector {

        private:
        unsigned int mCurrent;

        protected:
        const std::vector<unsigned int> mMapping;
        const std::vector<unsigned long> mStats;
        std::stack<unsigned int> mHistory;

        virtual bool select( unsigned int bit, const BitList & map ) const {

            return bit < mMapping.size() - 1 && !map.isSet( mMapping[ bit ] );
        }

        public:
        BitSelector ( const BitSelectStrategy & strategy ) :
            mCurrent( 0 ),
            mMapping( strategy.createMapping() ),
            mStats( strategy.getPopulationStats() ) {}
                
        unsigned int getNextBit ( const BitList & map ) {
            mHistory.push( mCurrent );

            while ( select( mCurrent, map ) ) {
                mCurrent++;
            }

            return mCurrent < mMapping.size() ? mMapping[ mCurrent++ ] : 0;
        }      

        bool checkNextBit ( const BitList & map ) const {
            unsigned int bit = mCurrent;

            while ( select( bit, map ) ) {
                bit++;
            }            

            return bit < mMapping.size();
        }
 
        void revert () {
            mCurrent = mHistory.top();
            mHistory.pop();
        }

        unsigned long getPopulationCoeff ( const BitList & map ) const {
            unsigned long coeff = 0;            

            for ( unsigned int bit = 0; bit < mStats.size(); bit++ ) {
                if ( map.isSet( bit ) ) {
                    coeff += mStats[ bit ];
                }
            }
            return coeff;
        }
    };
}

#endif // __DD_BIT_SELECTOR__
