/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_COMB_TEST_TRAVERSE_VISITOR__
#define __DD_COMB_TEST_TRAVERSE_VISITOR__

#include <deque>
#include <vector>

#include "DDTreeNode.h"
#include "DDSingleSetTraverseVisitor.h"
#include "DDSingleSetRay.h"

namespace double_description {

    class CombTestTraverseVisitor : public SingleSetTraverseVisitor {

        private:
        const TreeNode & mVerificator;

        public:
        CombTestTraverseVisitor( const TreeNode & tree,
                                 const unsigned short bits,
                                 const unsigned short plane,
                                 const unsigned short rows,
                                 const TreeNode & verificator,
                                 RayCache<SingleSetRay, BitPatternTreeCacheEntry<SingleSetRay> > & cache ) :
            SingleSetTraverseVisitor( tree, bits, plane, rows, cache ),
            mVerificator( verificator ) {}

        void onVisit ( const struct RayReference & current ) {

            if ( current.activeSet.getPopulationCount() == mMinCommonBits + 1 ) {
                SingleSetTraverseVisitor::onVisit( current );
            } else {
                RayRefVisitor::onVisit( current );
                mTree.preliminaryTraverse( current, mCandidates );   

                for ( std::deque<struct EdgeCandidate>::const_iterator it = mCandidates.begin(); it != mCandidates.end(); it++ ) {
                    const ExtVerificationCondition ver_cond( it->activeSet, it->first.activeSet,  it->second.activeSet );    

                    if ( !mVerificator.verificationTraverse( ver_cond ) ) {
                        create( *it );
                    }
                }    

                mCandidates.clear();         
            }
        }

        ~CombTestTraverseVisitor () {}

    };
}

#endif // __DD_COMB_TEST_TRAVERSE_VISITOR__
