
#ifndef __DD_COMBINATIONAL_SAMPLE_CREATOR__
#define __DD_COMBINATIONAL_SAMPLE_CREATOR__

#include <map>
#include <vector>
#include <iostream>
#include <algorithm>
#include <stdexcept>

#include "DDSampleCreator.h"

namespace double_description {

    class CombinationalSampleCreator : public SampleCreator {

        private:
        std::vector<bool> mCurrent;


        public:
        CombinationalSampleCreator ( const unsigned short blocks,
                                     const unsigned short minCommonBits,
                                     const unsigned short plane,
                                     const unsigned short bits ) :
            SampleCreator( blocks, minCommonBits ) {
            
                mCurrent.insert( mCurrent.begin(), bits, true );
                mCurrent.insert( mCurrent.begin(), plane - bits, false );
        }

        bool hasMorePermutations () {
            return std::next_permutation( mCurrent.begin(), mCurrent.end() );
        }


        const SamplesContainer createSamples( const std::vector<struct RayReference> &,
                                              const unsigned int plane,
                                              const unsigned int maxSamples ) const {

            SamplesContainer container( 1, mRegularThreshold, mInverseThreshold );
            const BitList sample( mCurrent );
               
            container.insert( sample, getInverseBasicPattern( sample, plane ) );
        
            return container;
        }

        const SamplesContainer createSamples( const std::vector<struct RayReference>& posEntries,
                                              const std::vector<struct RayReference>& negEntries,
                                              const unsigned int plane,
                                              const unsigned int maxSamples ) const {

            SamplesContainer container( 1, mRegularThreshold, mInverseThreshold ); 

            return container;
        }

        const std::vector<bool> & current () const {
            return mCurrent;
        }
    };
}

#endif // __DD_COMBINATIONAL_SAMPLE_CREATOR__
