/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_DEGENERACY_CALC_VISITOR__
#define __DD_DEGENERACY_CALC_VISITOR__

#include <vector>

#include "DDBitList.h"
#include "DDRayRefVisitor.h"

namespace double_description {

    class DegeneracyCalcVisitor : public RayRefVisitor {

        private:
        const unsigned short mDimension;
        double mDegeneracy;
        unsigned long mCurrentElement;

        protected:
        bool hasFinished () const {
            return true;
        }

        public:
        DegeneracyCalcVisitor( const unsigned short dim ) :
            RayRefVisitor(),
            mDimension( dim ) {}

        ~DegeneracyCalcVisitor () {}

        void onStart () {
            RayRefVisitor::onStart();

            mDegeneracy = 0.0;   
            mCurrentElement = 0;   
        }

        void onVisit ( const struct RayReference & ref ) {
            
            mDegeneracy = ( mDegeneracy * mCurrentElement + ref.degeneracy - mDimension ) / ( mCurrentElement + 1 );
            mCurrentElement++;
        }

        double degeneracy () const {
            return mDegeneracy;
        }
    };
}

#endif // __DD_DEGENERACY_CALC_VISITOR__
