
#ifndef __DD_DISTANCE_SYNTHESE_VANTAGE_POINT_VISITOR__
#define __DD_DISTANCE_SYNTHESE_VANTAGE_POINT_VISITOR__

#include "DDVantagePointVisitor.h"

#include "DDVantagePointPreliminaryCondition.h"

namespace double_description {

    class DistanceSyntheseVantagePointVisitor : public VantagePointVisitor {

        private:
        bool mMedianRound;

        protected:
        double mAverage;
        double mInvAverage;

        unsigned long mCount;
        unsigned long mInvCount;

        unsigned short mMinDistance;
        unsigned short mMinInvDistance;

        bool hasFinished () const {
            return !mMedianRound;
        }         

        virtual void onSave ( const struct RayReference & ref ) = 0;
        virtual void calcDistance () = 0;

        void update( unsigned short dist,
                     double & average,
                     unsigned long & count,
                     unsigned short & min_dist ) {

            average = ( average * count + dist )/( count + 1 );  
            if ( min_dist > dist ) {
                min_dist = dist;
            }
            count++;
        }

        public:
        DistanceSyntheseVantagePointVisitor ( const unsigned short depth,
                                              const unsigned short capacity,
                                              const unsigned short split ) :
            VantagePointVisitor( depth, capacity, split ) {}

        void before ( const RayReferenceContainer & current,
                      RayReferenceContainer * sets ) {

            VantagePointVisitor::before( current, sets );

            mMedianRound = true;

            mAverage = 0;
            mInvAverage = 0;

            mCount = 0;
            mInvCount = 0;
        }

        void onVisit ( const struct RayReference & ref ) {
            BitSplitVisitor::onVisit( ref );             

            if ( unlikely( mVantagePoint.isEmpty() ) ) {
                mVantagePoint = ref.activeSet;
                mMinDistance = mVantagePoint.getPopulationCount();
                mMinInvDistance = mVantagePoint.getCapacity() - mMinDistance;
            } else if ( mMedianRound ) {
                update( mVantagePoint.getCommonPopCnt( ref.activeSet ), mAverage, mCount, mMinDistance );
                update( mVantagePoint.getInvCommonPopCnt( ref.activeSet ), mInvAverage, mInvCount, mMinInvDistance );
            } else {
                onSave( ref );
            }
        }

        void onFinish () {
            BitSplitVisitor::onFinish();
            if ( mMedianRound ) {
                mMedianRound = false;
                calcDistance();
            }
        }    

    };
}

#endif // __DD_DISTANCE_SYNTHESE_VANTAGE_POINT_VISITOR__
