/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_DUAL_SET_RAY_ENUMERATOR__
#define __DD_DUAL_SET_RAY_ENUMERATOR__

#include "DDExtremeRayEnumerator.h"

#include "DDPartTestTraverseVisitor.h"

namespace double_description {

    class DualSetRayEnumerator : public ExtremeRayEnumerator<DualSetRay> {

        private:
        LinearDependenceCache & mDependenceCache;
        FaceProver & mProver;

        protected:
        unsigned long enumerate_pos_deg ( const TreeNode * const pos_deg_tree ) {

            PartTestTraverseVisitor verification_visitor( *pos_deg_tree, mMinCommonBits, mPlane, mSystem.getRows(), mCache, mDependenceCache, mProver );
            mNegativeEntries.iterate( verification_visitor );

            return verification_visitor.verified();
        }

        unsigned long enumerate_pos_nondeg ( const TreeNode * const pos_nondeg_tree ) {
            return DualSetRayEnumerator::enumerate_pos_deg( pos_nondeg_tree );
        }

        public:
        DualSetRayEnumerator ( const Matrix & system,
                               const unsigned short plane,
                               const BitPatternTreeCacheEntry<DualSetRay> & entry,
                               RayCache<DualSetRay, BitPatternTreeCacheEntry<DualSetRay> > & cache,
                               LinearDependenceCache & dependencies,
                               FaceProver & prover ) :
            ExtremeRayEnumerator( system, plane, entry, cache ),
            mDependenceCache( dependencies ),
            mProver( prover ) {}
    };
}

#endif // __DD_DUAL_SET_RAY_ENUMERATOR__
