/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_DUAL_SET_RAY_GENERATOR__
#define __DD_DUAL_SET_RAY_GENERATOR__

#include <vector>

#include "DDExtremeRayGenerator.h"

#include "DDDualSetRay.h"
#include "DDRepresentationVector.h"

#include "DDLinearDependenceCache.h"
#include "DDFaceProver.h"

namespace double_description {

	class DualSetRayGenerator : public ExtremeRayGenerator<DualSetRay> {

		private:
		LinearDependenceCache mDependenceCache;
        FaceProver mProver;

		RepresentationVector getInitialVector ( const Matrix & base, const unsigned long positivePlane, BitList & primaryActiveSet ) const;
		mpz_class* getPlaneCoeffs ( const RepresentationVector & vector, const BitList & primaryActiveSet, BitList & secondaryActiveSet, LinearDependenceEntry** & internalMap );
		const LinearDependenceEntry* getLinearDependenceEntry ( const BitList & primarySet, const unsigned long & secondaryBit );

        protected:
		const struct StepStats enumerate ( const unsigned long plane, BitPatternTreeCacheEntry<DualSetRay> & cache_entry );

		public:
#ifdef __COMPRESSION__
		DualSetRayGenerator ( const Matrix& original, const Matrix& system, const Matrix& transformator ) :
            ExtremeRayGenerator( original, system, transformator ),
#else
		DualSetRayGenerator ( const Matrix& system, const Matrix& transformator ) :
            ExtremeRayGenerator( system, transformator ),
#endif
            mDependenceCache ( system ),
            mProver( system.getRows(), system.getColumns() - 2 ) {}

		~DualSetRayGenerator () {}

		void calculateBase ();
	};
}

#endif // __DD_DUAL_SET_RAY_GENERATOR_
