
#include "DDFaceProver.h"

using namespace double_description;

FaceProver::FaceProver ( const unsigned short planes, const unsigned short rank ) :
	mPlanes( planes ),
	mBlocks( BitList::getBlocks( planes ) ),
	mTargetRank( rank ),
	mClearance( 0 ),
	mCurrentRank( 0 ),
	mRetainedPrimaryBits( mBlocks ),
	mEliminatedPrimaryBits( mBlocks ),
	mCandidateSecondaryBits( mBlocks ),
	mProvenSecondaryBits( mBlocks ),
	mCoeffLoaded( mBlocks ) {

	mWorkingSet = new BitRegister[ planes ];

	mCoeffCache = new mpz_t*[ planes ];

    mpz_init( gcd );
    mpz_init( ci_multiplicator );
    mpz_init( si_multiplicator );

	for ( unsigned int r_index = 0; r_index < planes; r_index++ ) {
		mCoeffCache[ r_index ] = new mpz_t[ planes ];

        for ( unsigned int c_index = 0; c_index < planes; c_index++ ) {
            mpz_init( mCoeffCache[ r_index ][ c_index ] );
        }
	}

	if ( mBlocks != INITIAL_REGISTER_WIDTH ) {
		for ( unsigned long plane = 0; plane < planes; plane++ ) {
			mWorkingSet[plane].resize( mBlocks );
		}
	}
}

FaceProver::FaceProver ( const FaceProver & other ) :
	mPlanes( other.mPlanes ),
	mBlocks( other.mBlocks ),
	mTargetRank( other.mTargetRank ),
	mClearance( 0 ),
	mCurrentRank( 0 ),
	mRetainedPrimaryBits( mBlocks ),
	mEliminatedPrimaryBits( mBlocks ),
	mCandidateSecondaryBits( mBlocks ),
	mProvenSecondaryBits( mBlocks ),
	mCoeffLoaded( mBlocks ) {

	mWorkingSet = new BitRegister[ mPlanes ];

	mCoeffCache = new mpz_t*[ mPlanes ];

	for ( unsigned int r_index = 0; r_index < mPlanes; r_index++ ) {
		mCoeffCache[ r_index ] = new mpz_t[ mPlanes ];

        for ( unsigned int c_index = 0; c_index < mPlanes; c_index++ ) {
            mpz_init( mCoeffCache[ r_index ][ c_index ] );
        }
	}

	if ( mBlocks != INITIAL_REGISTER_WIDTH ) {
		for ( unsigned long plane = 0; plane < mPlanes; plane++ ) {
			mWorkingSet[plane].resize( mBlocks );
		}
	}
}

FaceProver::~FaceProver () {
	delete[] mWorkingSet;

	for ( unsigned long plane = 0; plane < mPlanes; plane++ ) {
		delete[] mCoeffCache[plane];
	}

    delete[] mCoeffCache;
}
