
#include "DDIntermediateHybridTreeNode.h"
#include "DDPopTreePreliminaryCondition.h"

using namespace double_description;

PopTreeNode * IntermediateHybridTreeNode::createChildNode ( const std::vector<struct RayReference> & candidates,
															const BitList & map,
															const BitList & regPattern,
															const BitList & invPattern,
                                                            const unsigned short degeneracy,
															PopTreeCreatorUtils & utils,
															const unsigned int sampleID ) const {
	PopTreeNode * result;
    const PreliminaryCondition * cond = new PopTreePreliminaryCondition( map, mCond->getMinCommonBits(), regPattern, invPattern );

    HybridTreeCreatorUtils & hybrid_utils = static_cast<HybridTreeCreatorUtils &>( utils );

	if ( candidates.size() < utils.getMaxElementsPerLeaf() ) {
		result = new PopTreeLeafNode( candidates, map, cond, regPattern, invPattern, degeneracy );
	} else {
		if ( sampleID >= utils.samples().getSamplesCount() ) {
            IntermediatePopToBitSetNode * node = new IntermediatePopToBitSetNode( map, cond, regPattern, invPattern, degeneracy );
            node->partition( candidates, hybrid_utils );
			result = node;
		} else {
			IntermediateHybridTreeNode* node = new IntermediateHybridTreeNode( map, cond, regPattern, invPattern, degeneracy );
			node->partition( candidates, utils, sampleID );
			result = node;
		}				
	}

	return result;
}
