
#ifndef __DD_INTERMEDIATE_POP_TO_BIT_SET_NODE__
#define __DD_INTERMEDIATE_POP_TO_BIT_SET_NODE__

#include <sstream>

#include "DDBitList.h"

#include "DDPopTreeNode.h"
#include "DDIntermediateBitSetNode.h"

namespace double_description {

	class IntermediatePopToBitSetNode : public PopTreeNode, public IntermediateBitSetNode {

        protected:
        TreeNode * createChildNode ( const std::vector<struct RayReference> & candidates,
						    		 const BitList & map,
                                     const unsigned short degeneracy,
                                     BitSetTreeCreatorUtils & utils ) const;

		public:
		IntermediatePopToBitSetNode( const BitList & map,
                                     const PreliminaryCondition * const cond,
									 const BitList & regPattern,
									 const BitList & invPattern,
                                     const unsigned short degeneracy ) :
			TreeNode( map, cond, degeneracy ),
			PopTreeNode( map, cond, regPattern, invPattern, degeneracy ),
			IntermediateBitSetNode( map, cond, degeneracy ) {}

		virtual ~IntermediatePopToBitSetNode () {}

		void preliminaryTraverse ( const TreeNode * partner,
									std::vector<const void*> & candidates ) const {
			
	        if ( partner->getCondition()->holdsOnFace( mMap, mRegPattern, mInvPattern ) ) {
                partner->preliminaryTraverse( mSuperSet, candidates );
		        partner->preliminaryTraverse( mSubSet, candidates );
	        }
		}

		void preliminaryTraverse ( const RayReference * refs,
									const unsigned int refsCount,
									const PreliminaryCondition & condition,
									std::vector<const void*> & candidates ) const {

			if ( condition.holdsOnFace( mMap, mRegPattern, mInvPattern ) ) {
				mSuperSet->preliminaryTraverse( refs, refsCount, condition, candidates );
				mSubSet->preliminaryTraverse( refs, refsCount, condition, candidates );
			} 			
		}

		bool verificationTraverse ( const VerificationCondition & condition ) const;

        bool isRedundant () const {
            return false;
        }

        PopTreeNode * getNextNonRedundant () {
            return this;
        }
/*
		const std::string toString () const {
		    std::stringstream result;

		    const TreeNode *node;

		    node = this;

		    result << "p" << static_cast<const void*>( node ) << " [label=" << getRaysCount() << "];\n";

		    return result.str();
		} 
*/	};
}

#endif // __DD_INTERMEDIATE_POP_TO_BIT_SET_NODE__
