
#ifndef __DD_INTERMEDIATE_POPULATION_NODE__
#define __DD_INTERMEDIATE_POPULATION_NODE__

#include "DDPopTreeNode.h"
#include "DDPopTreeLeafNode.h"
#include "DDPreliminaryCondition.h"
#include "DDVerificationCondition.h"
#include "DDSamplesContainer.h"
#include "DDPopTreeCreatorUtils.h"

namespace double_description {

	struct PatternBlock {
		const BitList* samples;
		unsigned int count;
		unsigned int pop_req;		
		unsigned int inv_pop_req;		
	};

	class IntermediatePopulationNode : public PopTreeNode {

		private:
		PopTreeNode * mOver;
		PopTreeNode * mUnder;
		PopTreeNode * mDoubleOver; 

		protected:

		virtual PopTreeNode * createChildNode ( const std::vector<struct RayReference> & candidates,
						    					const BitList & map,
						    					const BitList & regPattern,
						    					const BitList & invPattern,
                                                const unsigned short degeneracy,
						    					PopTreeCreatorUtils & utils,
						    					const unsigned int sampleID ) const;

		public:
		IntermediatePopulationNode ( const BitList & map,
                                     const PreliminaryCondition * const cond,
									 const unsigned int samplesCount );

		IntermediatePopulationNode ( const BitList & map,
                                     const PreliminaryCondition * const cond,
									 const BitList & regPattern,
									 const BitList & invPattern,
                                     const unsigned short degeneracy ); 

		virtual ~IntermediatePopulationNode () {
            delete mOver;
            delete mUnder;
            delete mDoubleOver;
        }

		void partition ( const std::vector<struct RayReference> & refs,
			                PopTreeCreatorUtils & utils,
							const unsigned int sampleID );

		void preliminaryTraverse ( const TreeNode * partner,
									std::vector<const void*> & candidates ) const;

		void preliminaryTraverse ( const RayReference * refs,
									const unsigned int refsCount,
									const PreliminaryCondition & condition,
									std::vector<const void*> & candidates ) const;

		bool verificationTraverse ( const VerificationCondition & condition ) const;

        unsigned long getSubNodesCount () const;
        unsigned long getRaysCount () const;
        unsigned long getLeafNodesCount () const;
        
        void enumerateLeafNodes( std::vector<const TreeNode*> & container ) const;
        void enumeratePopTreeNodes( std::vector<const PopTreeNode*> & container ) const;

        bool isEmpty () const {
            return false;
        }

        bool isRedundant () const {
            return ( mOver->isEmpty() && mUnder->isEmpty() ) ||
                   ( mOver->isEmpty() && mDoubleOver->isEmpty() ) ||
                   ( mDoubleOver->isEmpty() && mUnder->isEmpty() );
        }

        PopTreeNode * getNextNonRedundant ();

        void optimize ();

        const std::string toString () const;
	};
}

#endif // __DD_INTERMEDIATE_POPULATION_NODE__
