/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_LINEAR_DEPENDENCE_ENTRY__
#define __DD_LINEAR_DEPENDENCE_ENTRY__

#include <gmp.h>
#include <gmpxx.h>

#include <string.h>

#include "DDBitList.h"
#include "DDVector.h"

namespace double_description {

	class LinearDependenceEntry : public Vector {

		private:
		BitList mMap;
		unsigned char* mCoeffMapping;
        unsigned short mRefs;

		public:
		LinearDependenceEntry ( const unsigned long blocks, const unsigned long dim ) :
			Vector( 1 ),
			mMap( blocks ),
            mRefs( 0 ) {
		
			mCoeffMapping = new unsigned char[dim];
		}

		LinearDependenceEntry ( const unsigned long blocks, const unsigned long dim, const mpz_class & factor ) :
			Vector( 1, factor ),
			mMap( blocks ),
            mRefs( 0 ) {

			mCoeffMapping = new unsigned char[dim];
		}

		~LinearDependenceEntry () {
			delete[] mCoeffMapping;
		}

		void insert ( const unsigned long position, const mpz_class & coeff ) {
			mMap.setBit( position );
			mCoeffMapping[position] = data.size();
			data.push_back( coeff );
		}

		const BitList & getMap () const {
			return mMap;
		}

		const mpz_class& getCoeff ( const unsigned long position ) const {
			return data[ mCoeffMapping[position] ];
		}

		void setScalingFactor ( const mpz_class & factor ) {
			data[0] = factor;
		} 

		const mpz_class& getScalingFactor () const {
			return data[0];
		}

        void reference () {
            mRefs++;
        }

        void dereference () {
            mRefs--;
        }

        bool is_referenced () const {
            return mRefs != 0;
        }        

        const std::string toString () const {
            return mMap.toString();
        }
	};
}


#endif // __DD_LINEAR_DEPENDENCE_ENTRY__
