/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_PART_TEST_TRAVERSE_VISITOR__
#define __DD_PART_TEST_TRAVERSE_VISITOR__

#include "DDPreliminaryTraverseVisitor.h"
#include "DDDualSetRay.h"
#include "DDLinearDependenceCache.h"
#include "DDFaceProver.h"
#include "DDDualSetFace.h"

namespace double_description {

    class PartTestTraverseVisitor : public PreliminaryTraverseVisitor<DualSetRay> {

        private:
        LinearDependenceCache & mDependenceCache;
        FaceProver & mProver;
        const unsigned short mBlocks;
    
        public:
        PartTestTraverseVisitor( const TreeNode & tree,
                                 const unsigned short bits,
                                 const unsigned short plane,
                                 const unsigned short rows,
                                 RayCache<DualSetRay, BitPatternTreeCacheEntry<DualSetRay> > & cache,
                                 LinearDependenceCache & dependencies,
                                 FaceProver & prover ) :
            PreliminaryTraverseVisitor( tree, bits, plane, rows, cache ),
            mDependenceCache( dependencies ),
            mProver( prover ),
            mBlocks( BitList::getBlocks( rows )) {}

        void onVisit ( const struct RayReference & current ) {
            RayRefVisitor::onVisit( current );
            mTree.preliminaryTraverse( current, mCandidates );   

            for ( std::deque<struct EdgeCandidate>::const_iterator it = mCandidates.begin(); it != mCandidates.end(); it++ ) {
	            const DualSetRay * positive = static_cast<const DualSetRay*>( it->first.ray );
			    const DualSetRay * negative = static_cast<const DualSetRay*>( it->second.ray );                     

		        if ( mProver.isFace ( positive, negative ) ) {
			        DualSetFace face( positive, negative, mBlocks );
			        face.setPrimaryActiveSet( mProver.getRetainedPrimaryBits(), mPlane );
			        face.setSecondaryActiveSet( mProver.getProvenSecondaryBits() );
			        mCache.insert( face.getIntersection( mPlane, mRows, mDependenceCache ), 1 );

    #ifdef __STATS_OUTPUT__
                    mVerified++;
    #endif
		        }          
            }
            mCandidates.clear();
        }

        ~PartTestTraverseVisitor () {}
    };
}

#endif // __DD_PART_TEST_TRAVERSE_VISITOR__
