
#ifndef __DD_POP_TREE_LEAF_NODE__
#define __DD_POP_TREE_LEAF_NODE__

#include "DDLogging.h"

#include "DDBitList.h"
#include "DDTreeNode.h"
#include "DDPopTreeNode.h"
#include "DDLeafNode.h"
#include "DDPreliminaryCondition.h"
#include "DDVerificationCondition.h"

namespace double_description {

	class PopTreeLeafNode : public LeafNode, public PopTreeNode {

		public:
		PopTreeLeafNode ( const std::vector<struct RayReference> & refs,
							const BitList & map,
                            const PreliminaryCondition * const cond,
							const BitList & regPattern,
							const BitList & invPattern,
                            const unsigned short degeneracy ) : 
			TreeNode( map, cond, degeneracy ),
			LeafNode( refs, map, cond, degeneracy ),
			PopTreeNode( map, cond, regPattern, invPattern, degeneracy ) {}

		virtual ~PopTreeLeafNode() {}

		void preliminaryTraverse ( const TreeNode * partner,
									std::vector<const void*> & candidates ) const {

            if ( partner->getCondition()->holdsOnFace( mMap, mRegPattern, mInvPattern ) ) {
	            partner->preliminaryTraverse( mRefs, mRefsCount, *mCond, candidates );
            }
		}

		void preliminaryTraverse ( const RayReference * refs,
									const unsigned int refsCount,
									const PreliminaryCondition & condition,
									std::vector<const void*> & candidates ) const {

			if ( condition.holdsOnFace( mMap, mRegPattern, mInvPattern ) ) {
				LeafNode::preliminaryTraverse( refs, refsCount, candidates );
			}
		}

		bool verificationTraverse ( const VerificationCondition & condition ) const {
			
			if ( condition.holdsOnFace( mMap, mRegPattern, mInvPattern ) ) {
				return checkRays( condition );
			} else {
				return false;
			}
		}

        bool isRedundant () const {
            return false;
        }

        PopTreeNode * getNextNonRedundant () {
            return this;
        }
	};

}

#endif // __DD_POP_TREE_LEAF_NODE__
