
#ifndef __DD_POP_TREE_NODE__
#define __DD_POP_TREE_NODE__

#include "DDBitList.h"
#include "DDTreeNode.h"
#include "DDPreliminaryCondition.h"
#include "DDVerificationCondition.h"

namespace double_description {

	class PopTreeNode : virtual public TreeNode {

		protected:
		const BitList mRegPattern;
		const BitList mInvPattern;

		public:
		PopTreeNode ( const BitList & map,
                      const PreliminaryCondition * const cond,
                      const BitList & regPattern,
                      const BitList & invPattern,
                      const unsigned short degeneracy ) : 
			TreeNode( map, cond, degeneracy ),
			mRegPattern( regPattern ),
			mInvPattern( invPattern ) {}

		virtual ~PopTreeNode() {}

		const BitList & getRegularPattern () const {
			return mRegPattern;
		}

		const BitList & getInversePattern () const {
			return mInvPattern;
		}

        virtual void enumeratePopTreeNodes( std::vector<const PopTreeNode*> & container ) const {
            container.push_back( this );
        }

        virtual bool isRedundant () const = 0;
        virtual PopTreeNode * getNextNonRedundant () = 0;

        virtual void optimize () {
            /* Do nothing on default. */
        }
	};

    struct ComparatorMostPopulatedFirst {

        bool operator() ( const PopTreeNode * lhs, const PopTreeNode * rhs ) const {
            return lhs->getRegularPattern().getPopulationCount() + lhs->getInversePattern().getPopulationCount() >
                   rhs->getRegularPattern().getPopulationCount() + rhs->getInversePattern().getPopulationCount();
        }
    };
}

#endif // __DD_POP_TREE_NODE__
