
#ifndef __DD_POPULATION_TREE_CACHE_ENTRY__
#define __DD_POPULATION_TREE_CACHE_ENTRY__

#include <limits.h>
#include <vector>

#include "DDStats.h"

#include "DDPopulationTreeNode.h"
#include "DDTreeRayContainer.h"
#include "DDCombinationalTest.h"

#include "DDIntermediatePopulationNode.h"
#include "DDIntermediateHybridTreeNode.h"
#include "DDPopTreeVerificationCondition.h"
#include "DDPopTreePreliminaryCondition.h"
#include "DDExtPopTreeVerificationCondition.h"

#include "DDSamplesContainer.h"
#include "DDRegularStaticSampleCreator.h"
#include "DDMostCommonFirstDynamicSampleCreator.h"
#include "DDLeastCommonFirstDynamicSampleCreator.h"
#include "DDMostAverageFirstDynamicSampleCreator.h"
#include "DDCombinationalSampleCreator.h"

#include "DDHybridTreeCreatorUtils.h"

#include "DDMostCommonBitFirstStrategy.h"
#include "DDMostAverageBitFirstStrategy.h"
#include "DDLeastAverageBitFirstStrategy.h"
#include "DDLeastCommonBitFirstStrategy.h"
#include "DDFirstAvailableBitStrategy.h"

namespace double_description {

	template <class RAY_TYPE>
	class PopulationTreeCacheEntry {

		private:
		static const unsigned long PATTERN_SET_A[];
		static const unsigned short PATTERN_SET_A_SIZE;

		static const unsigned long PATTERN_SET_B[];
		static const unsigned short PATTERN_SET_B_SIZE;

		static const unsigned long PATTERN_SET_C[];
		static const unsigned short PATTERN_SET_C_SIZE;

		static const unsigned long PATTERN_SET_D[];
		static const unsigned short PATTERN_SET_D_SIZE;

		static const unsigned long PATTERN_SET_E[];
		static const unsigned short PATTERN_SET_E_SIZE;

		unsigned short mPlane;
		unsigned short mBlocks;

		const BitList* mPatternSet;
		unsigned short mPatternSetSize;
		
        unsigned short mMinCommonBits;
		unsigned short mPatternThreshold;
		unsigned short mInverseThreshold;

		PopulationTreeNode<RAY_TYPE> mPositive;
		std::vector<struct RayReference> mPositiveEntries;
		BitList mPositiveMap;

		PopulationTreeNode<RAY_TYPE> mNegative;
		std::vector<struct RayReference> mNegativeEntries;
		BitList mNegativeMap;

		PopulationTreeNode<RAY_TYPE> mZero;
		std::vector<struct RayReference> mZeroEntries;
		BitList mZeroMap;
        BitList mCutMap;

		unsigned long mPositiveCount;
		unsigned long mNegativeCount;

		CombinationalTest<RAY_TYPE, PopulationTreeNode<RAY_TYPE> > mCombiTest;

		static const BitList * createPatterns ( const unsigned long pattern[], const unsigned short count, const unsigned short plane ) {
			
			BitList * list = new BitList[ 2*count ];
			const unsigned short blocks = BitList::getBlocks( plane + 1 );

			for ( unsigned short index = 0; index < count; index++ ) {
				list[2*index].init( blocks, pattern[ index ] );
				list[2*index+1].init( blocks, ~pattern[ index ] );

				list[2*index].unsetBit( plane );
				list[2*index+1].unsetBit( plane );

				list[2*index].unsetBits( plane, blocks * __BLOCK_SIZE__ );
				list[2*index+1].unsetBits( plane, blocks * __BLOCK_SIZE__ );
			}

			return list;
		}

		static const BitList * copyPatterns ( const BitList * const original, const unsigned short size ) {
			BitList * list = new BitList[ size ];

			for ( unsigned short index = 0; index < size; index++ ) {
				list[index] = original[index];
			}

			return list;			
		}

		unsigned long combine ( const PopulationTreeNode<RAY_TYPE>* positive, const PopulationTreeNode<RAY_TYPE>* negative,
						std::vector<const RAY_TYPE*> & buffer, const unsigned short minCommonBits ) {
		
		/*	const TreeRayContainer<RAY_TYPE>* pContainer = positive->getContainer();
			const TreeRayContainer<RAY_TYPE>* nContainer = negative->getContainer(); 

			const BitList & positive_union = pContainer->getPositiveUnion();
			const BitList & negative_union = nContainer->getNegativeUnion(); */

			const BitList & positive_union = positive->getMap();
			const BitList & negative_union = negative->getMap();

			unsigned long adj_tests = 0;

		/*	BitList cut = positive_union & negative_union;

			if ( cut.getPopulationCount() >= minCommonBits ) {
		
				const std::vector<const RAY_TYPE*> & positive_rays = pContainer->getPositiveRays();
				const std::vector<const RAY_TYPE*> & negative_rays = nContainer->getNegativeRays(); */

				const std::vector<const RAY_TYPE*> & positive_rays = positive->getRays();
				const std::vector<const RAY_TYPE*> & negative_rays = negative->getRays();

				for ( unsigned int p_index = 0; p_index < positive_rays.size(); p_index++ ) {
										
					if ( negative_union.getCommonPopCnt( positive_rays[ p_index ]->getActiveSet() ) >= minCommonBits ) {

						adj_tests += negative_rays.size();

						for ( unsigned int n_index = 0; n_index < negative_rays.size(); n_index++ ) {
					
#ifdef __COMBINATIONAL_TEST__
							if ( mCombiTest.adjacencyTest( positive_rays[p_index]->getActiveSet(), negative_rays[n_index]->getActiveSet(), positive, negative, minCommonBits ) ) {
								buffer.push_back( positive_rays[p_index] );
								buffer.push_back( negative_rays[n_index] ); 
							}
#else // __ALGEBRAIC_TEST__
							register const unsigned long common_bits = positive_rays[p_index]->getActiveSet().getCommonPopCnt( negative_rays[n_index]->getActiveSet() );

							if ( __builtin_expect ( common_bits >= minCommonBits, 0 ) ) {
								buffer.push_back( positive_rays[p_index] );
								buffer.push_back( negative_rays[n_index] ); 
							}
#endif // __COMBINATIONAL_TEST__
						}				
					}
	//			}
			}	

			return adj_tests;
		}

		unsigned long traverse ( const BitSetTreeNode<RAY_TYPE>* positive, const BitSetTreeNode<RAY_TYPE>* negative, std::vector<const RAY_TYPE*> & buffer, const unsigned short minCommonBits ) {
			
			unsigned long result = 0;

			if ( positive->getMap().getCommonPopCnt ( negative->getMap() ) >= minCommonBits ) {
				if ( positive->isLeaf() ) {
					if ( negative->isLeaf() ) {
						result = combine( positive, negative, buffer, minCommonBits );
					} else {
						result = traverse( positive, negative->getSuperSet(), buffer, minCommonBits );
						result += traverse( positive, negative->getSubSet(), buffer, minCommonBits );
					}
				} else {
					if ( negative->isLeaf() ) {
						result = traverse( positive->getSuperSet(), negative, buffer, minCommonBits );
						result += traverse( positive->getSubSet(), negative, buffer, minCommonBits );
					} else {
						result = traverse( positive->getSuperSet(), negative->getSuperSet(), buffer, minCommonBits );
						result += traverse( positive->getSuperSet(), negative->getSubSet(), buffer, minCommonBits );
						result += traverse( positive->getSubSet(), negative->getSuperSet(), buffer, minCommonBits );
						result += traverse( positive->getSubSet(), negative->getSubSet(), buffer, minCommonBits );
					}
				}
			}

			return result;
		}

		unsigned long combine ( const BitSetTreeNode<RAY_TYPE>* pContainer, const BitSetTreeNode<RAY_TYPE>* nContainer,
						std::vector<const RAY_TYPE*> & buffer, const unsigned short minCommonBits ) {
		
			const std::vector<const RAY_TYPE*> p_vec = pContainer->getRays();
			const std::vector<const RAY_TYPE*> n_vec = nContainer->getRays();

			unsigned long adj_tests = 0;

			for ( unsigned int p_index = 0; p_index < p_vec.size(); p_index++ ) {

				if ( nContainer->getMap().getCommonPopCnt( p_vec[p_index]->getActiveSet() ) >= minCommonBits ) {

					for ( unsigned int n_index = 0; n_index < n_vec.size(); n_index++ ) {

						if ( mCombiTest.adjacencyTest( p_vec[p_index]->getActiveSet(), n_vec[n_index]->getActiveSet(), pContainer, nContainer, minCommonBits ) ) {

							buffer.push_back( p_vec[p_index] );
							buffer.push_back( n_vec[n_index] ); 					
						}		
					}	

					adj_tests += n_vec.size();
				}
			}	

			return adj_tests;
		}

		unsigned long traverse ( PopulationTreeNode<RAY_TYPE>* positive, PopulationTreeNode<RAY_TYPE>* negative, std::vector<const RAY_TYPE*> & buffer, const unsigned short minCommonBits ) {

			unsigned long adj_tests = 0;

			if ( positive->areCombinable( negative ) && positive->getMap().getCommonPopCnt( negative->getMap() ) >= minCommonBits ) {

				if ( positive->isLeaf() && negative->isLeaf() ) {
					adj_tests = traverse( positive->getBitPatternTree(), negative->getBitPatternTree(), buffer, minCommonBits );
				} else if ( positive->isLeaf() ) {

					adj_tests += traverse( positive, negative->getPatternUnderNode(), buffer, minCommonBits );
					adj_tests += traverse( positive, negative->getInverseUnderNode(), buffer, minCommonBits );
					adj_tests += traverse( positive, negative->getDoubleOverNode(), buffer, minCommonBits );

				} else if ( negative->isLeaf() ) {

					adj_tests = traverse( positive->getPatternUnderNode(), negative, buffer, minCommonBits );
					adj_tests += traverse( positive->getInverseUnderNode(), negative, buffer, minCommonBits );
					adj_tests += traverse( positive->getDoubleOverNode(), negative, buffer, minCommonBits );		
		
				} else {

					adj_tests = traverse( positive->getPatternUnderNode(), negative->getPatternUnderNode(), buffer, minCommonBits );
					adj_tests += traverse( positive->getPatternUnderNode(), negative->getDoubleOverNode(), buffer, minCommonBits );

					adj_tests += traverse( positive->getInverseUnderNode(), negative->getInverseUnderNode(), buffer, minCommonBits );
					adj_tests += traverse( positive->getInverseUnderNode(), negative->getDoubleOverNode(), buffer, minCommonBits );

					adj_tests += traverse( positive->getDoubleOverNode(), negative->getPatternUnderNode(), buffer, minCommonBits );
					adj_tests += traverse( positive->getDoubleOverNode(), negative->getInverseUnderNode(), buffer, minCommonBits );
					adj_tests += traverse( positive->getDoubleOverNode(), negative->getDoubleOverNode(), buffer, minCommonBits );
				}
			} 

			return adj_tests;
		}
		
		public:
		PopulationTreeCacheEntry ( const unsigned short plane, const unsigned short minCommonBits ) :
			mPlane( plane ),
			mBlocks( BitList::getBlocks( plane + 1 ) ),
			mPatternSet( createPatterns( PATTERN_SET_E, PATTERN_SET_E_SIZE, plane ) ),
			mPatternSetSize( PATTERN_SET_E_SIZE ),
            mMinCommonBits( minCommonBits ),
			mPatternThreshold( minCommonBits % 2 == 0 ? minCommonBits/2 + 1 : (minCommonBits + 1)/2 ),
			mInverseThreshold( minCommonBits % 2 == 0 ? minCommonBits/2 : (minCommonBits + 1)/2 ),
			mPositive( BitList( mBlocks, ULONG_MAX ), BitList( mBlocks, ULONG_MAX ), mPatternSet[0] ),
			mPositiveMap( mBlocks ),
			mNegative( BitList( mBlocks, ULONG_MAX ), BitList( mBlocks, ULONG_MAX ), mPatternSet[0] ),
			mNegativeMap( mBlocks ),
			mZero( BitList( mBlocks, ULONG_MAX ), BitList( mBlocks, ULONG_MAX ), mPatternSet[0] ),
			mZeroMap( mBlocks ),
            mCutMap( mBlocks, 0 ),
			mPositiveCount( 0 ),
			mNegativeCount( 0 ),
			mCombiTest( &mPositive, &mNegative, &mZero, mPatternThreshold, mInverseThreshold, mBlocks, mPatternSet, mPatternSetSize ) {

	//		mTree.partition( SplitCriteria<RAY_TYPE>( PATTERN_SET_A_SIZE, maxSeparationSteps ), mLeafs, mBlocks, mPatternSet );

            mCutMap.setBits( 0, mPlane );
		}

		PopulationTreeCacheEntry ( const PopulationTreeCacheEntry<RAY_TYPE> & other ) :
			mPlane( other.mPlane ),
			mBlocks( other.mBlocks ),
			mPatternSet( copyPatterns( other.mPatternSet, 2*other.mPatternSetSize ) ),
			mPatternSetSize( other.mPatternSetSize ),
            mMinCommonBits( other.mMinCommonBits ),
			mPatternThreshold( other.mPatternThreshold ),
			mInverseThreshold( other.mInverseThreshold ),
			mPositive( other.mPositive ),
			mPositiveMap( other.mPositiveMap ),
			mNegative( other.mNegative ),
			mNegativeMap( other.mPositiveMap ),
			mZero( other.mZero ),
			mZeroMap( other.mZeroMap ),
            mCutMap( other.mCutMap ),
			mPositiveCount( other.mPositiveCount ),
			mNegativeCount( other.mNegativeCount ),
			mCombiTest(  &mPositive, &mNegative, &mZero, mPatternThreshold, mInverseThreshold, mBlocks, mPatternSet, mPatternSetSize ) {}

		~PopulationTreeCacheEntry () {
			delete[] mPatternSet;
		}
		
		void insertPositive ( const RAY_TYPE* const ray ) {
			mPositiveCount++;
	//		mPositive.insert( ray, /*POSITIVE,*/ mPatternThreshold, mInverseThreshold );
			mPositiveEntries.push_back( RayReference( static_cast<const void*>( ray ), &ray->getActiveSet() ));
			mPositiveMap |= ray->getActiveSet();
            
			insertZero( ray );
		}

		void insertZero ( const RAY_TYPE* const ray ) {
	//		mZero.insert( ray, /*ZERO,*/ mPatternThreshold, mInverseThreshold );
    
            if ( mCutMap.getCommonPopCnt( ray->getActiveSet() ) > mMinCommonBits + 1 ) {
			    mZeroEntries.push_back( RayReference( static_cast<const void*>( ray ), &ray->getActiveSet() ));
			    mZeroMap |= ray->getActiveSet();
            }
		}

		void insertNegative ( const RAY_TYPE* const ray ) {
			mNegativeCount++;
	//		mNegative.insert( ray, /*NEGATIVE,*/ mPatternThreshold, mInverseThreshold );
			mNegativeEntries.push_back( RayReference( static_cast<const void*>( ray ), &ray->getActiveSet() ));
			mNegativeMap |= ray->getActiveSet();

			insertZero( ray );
		}

		unsigned int getPositiveCount () const {
			return mPositiveCount;
		}

		unsigned long getNegativeCount () const {
			return mNegativeCount;
		}

		const struct StepStats fillCandidatesBuffer ( std::vector<const RAY_TYPE*> & buffer,
                                                      const unsigned short minCommonBits ) {

            struct StepStats stats;

            const MostCommonFirstDynamicSampleCreator sample_creator( mBlocks, minCommonBits, 0.1, 0.2, 0.9 );
            SamplesContainer container = sample_creator.createSamples( mPositiveEntries, mNegativeEntries, mPlane, 40 );

            const MostCommonFirstDynamicSampleCreator ver_creator( mBlocks, minCommonBits, 0.1, 0.1, 0.9 );
      //      const RegularStaticSampleCreator ver_creator( mBlocks, minCommonBits, 0.1 );
            

            BitList full_list = container.createInitialPattern();

#ifdef __HYBRID_TREES__
			IntermediateHybridTreeNode positive_tree = IntermediateHybridTreeNode( mPositiveMap, new PopTreePreliminaryCondition( mPositiveMap, minCommonBits, full_list, full_list ), container.getSamplesCount() );
			IntermediateHybridTreeNode negative_tree = IntermediateHybridTreeNode( mNegativeMap, new PopTreePreliminaryCondition( mNegativeMap, minCommonBits, full_list, full_list ), container.getSamplesCount() );
#else
			IntermediatePopulationNode positive_tree = IntermediatePopulationNode( mPositiveMap, new PopTreePreliminaryCondition( mPositiveMap, minCommonBits, full_list, full_list ), container.getSamplesCount() );
			IntermediatePopulationNode negative_tree = IntermediatePopulationNode( mNegativeMap, new PopTreePreliminaryCondition( mPositiveMap, minCommonBits, full_list, full_list ), container.getSamplesCount() );	
#endif

            BitSelector positive_selector( MostCommonBitFirstStrategy ( mPositiveEntries, mPlane ));
            BitSelector negative_selector( MostCommonBitFirstStrategy ( mNegativeEntries, mPlane ));
           
            HybridTreeCreatorUtils positive_utils( mPositiveEntries.size()/4096 + 8, mCutMap, positive_selector, mPlane, container );
            HybridTreeCreatorUtils negative_utils( mNegativeEntries.size()/4096 + 8, mCutMap, negative_selector, mPlane, container );

			positive_tree.partition( mPositiveEntries, positive_utils, 0 );
			negative_tree.partition( mNegativeEntries, negative_utils, 0 );

            positive_tree.optimize();
            negative_tree.optimize();

#ifdef __STATS_OUTPUT__
            std::stringstream positive_file;
            positive_file << "positive_" << mPlane << ".dot";
            positive_tree.toFile( positive_file.str() );

            std::stringstream negative_file;
            negative_file << "negative_" << mPlane << ".dot";
            negative_tree.toFile( negative_file.str() ); 

            PreliminaryCondition::start_new_stat_session();
#endif

			std::vector<const void*> candidates;

	        START_TIMER( PRELIMINARY_PHASE );

			positive_tree.preliminaryTraverse( &negative_tree, candidates );

	        STOP_TIMER( PRELIMINARY_PHASE );
            stats.eliminationTime = TIMER_VALUE( PRELIMINARY_PHASE );
            stats.verificationCandidates = candidates.size() / 2;

#ifdef __COMBINATIONAL_TEST__
            START_TIMER( VERIFICATION_PHASE );

    #ifdef __STATS_OUTPUT__
            VerificationCondition::start_new_stat_session();
            LeafNode::start_new_stat_session();
    #endif

            std::vector<struct PopulationEntry> population = createStats( candidates );
            SamplesContainer ver_container = ver_creator.createSamples( mZeroEntries, population, mPlane, 40 );
    #ifdef __HYBRID_TREES__
			IntermediateHybridTreeNode zero_tree = IntermediateHybridTreeNode( mZeroMap, new PopTreePreliminaryCondition( mZeroMap, minCommonBits, full_list, full_list ), ver_container.getSamplesCount() ); 
    #else
			IntermediatePopulationNode zero_tree = IntermediatePopulationNode( mZeroMap, new PopTreePreliminaryCondition( mPositiveMap, minCommonBits, full_list, full_list ), ver_container.getSamplesCount() );		
    #endif
            BitSelector zero_selector( MostCommonBitFirstStrategy ( mZeroEntries, mPlane ));
            HybridTreeCreatorUtils zero_utils( mZeroEntries.size()/4096 + 8, mCutMap, zero_selector, mPlane, ver_container );
            zero_tree.partition( mZeroEntries, zero_utils, 0 );
            zero_tree.optimize();

    #ifdef __STATS_OUTPUT__
            std::stringstream ver_tree_file;
            ver_tree_file << "ver_hyb_tree_" << mPlane << ".dot";
            zero_tree.toFile( ver_tree_file.str() );
    #endif

            findAdjacentRays( candidates, zero_tree, ver_container, buffer );

            STOP_TIMER( VERIFICATION_PHASE );
            stats.verificationTime += TIMER_VALUE( VERIFICATION_PHASE );

    #ifdef __STATS_OUTPUT__
            stats.elimOps = VerificationCondition::OPS;
            stats.verOps = PreliminaryCondition::OPS;
            stats.leafNodeVisits = LeafNode::LEAF_NODE_VISITS;
            stats.totalLeafNodes = zero_tree.getLeafNodesCount();
            stats.averageMapPopulation = LeafNode::AVERAGE_MAP_POPULATION;
            stats.averageRaysPerLeaf = LeafNode::AVERAGE_RAYS_PER_LEAF;
    #endif
#endif

			return stats;
		}

        std::vector<struct PopulationEntry> createStats ( const std::vector<const void*> & candidates ) const {
            std::vector<struct PopulationEntry> stats;

            for ( unsigned int index = 0; index < mPlane; index++ ) {
                stats.push_back( PopulationEntry( index ) );
            }

            BitList temp( mBlocks ); 

            for ( unsigned long index = 0; index < candidates.size(); index += 2 ) {
				
				const RAY_TYPE* first = static_cast<const RAY_TYPE*>( candidates[index] );
				const RAY_TYPE* second = static_cast<const RAY_TYPE*>( candidates[index+1] );

				temp = first->getActiveSet();
				temp &= second->getActiveSet();

                for ( unsigned int index = 0; index < stats.size(); index++ ) {
                    if ( temp.isSet( index ) ) {
                        stats[ index ].count++;
                    }
                }           
            }

            return stats;
        } 

        void findAdjacentRays ( const std::vector<const void*> & candidates,
                                const PopTreeNode & current,
                                SamplesContainer & samples,
                                std::vector<const RAY_TYPE*> & buffer ) const {

            BitList temp( mBlocks ); 

#ifdef __STATS_OUTPUT__
           std::vector<unsigned long> population( mPlane, 0 );
#endif

			for ( unsigned long index = 0; index < candidates.size(); index += 2 ) {
				
				const RAY_TYPE* first = static_cast<const RAY_TYPE*>( candidates[index] );
				const RAY_TYPE* second = static_cast<const RAY_TYPE*>( candidates[index+1] );

				temp = first->getActiveSet();
				temp &= second->getActiveSet();

				const RAY_TYPE* positive = first->isInside( mPlane ) ? first : second;
				const RAY_TYPE* negative = first->isInside( mPlane ) ? second : first;

#ifdef __COMBINATIONAL_TEST__
				BitList reg_pattern = samples.createInitialPattern();
				BitList inv_pattern = samples.createInitialPattern();

                samples.refinePattern( reg_pattern, inv_pattern, temp );

				const ExtPopTreeVerificationCondition ver_cond( temp, positive->getActiveSet(), negative->getActiveSet(), reg_pattern, inv_pattern );
                
                if ( !current.verificationTraverse( ver_cond ) ) {
					buffer.push_back( positive );
					buffer.push_back( negative );                    
                } 

#else // __ALGEBRAIC_TEST__

				buffer.push_back( positive );
				buffer.push_back( negative );

#endif // __COMBINATIONAL_TEST__

			}         
        }

#ifdef __HYBRID_TREES__
		const struct StepStats fillCandidatesBufferSim ( std::vector<const RAY_TYPE*> & buffer,
                                                      const unsigned short minCommonBits ) {

            struct StepStats stats;

            const MostCommonFirstDynamicSampleCreator sample_creator( mBlocks, minCommonBits, 0.1, 0.2, 0.9 );
            SamplesContainer container = sample_creator.createSamples( mPositiveEntries, mNegativeEntries, mPlane, 40 );

            CombinationalSampleCreator ver_creator( mBlocks, minCommonBits, mPlane, mPlane/2 );

            BitList full_list = container.createInitialPattern();

			IntermediateHybridTreeNode positive_tree = IntermediateHybridTreeNode( mPositiveMap, new PopTreePreliminaryCondition( mPositiveMap, minCommonBits, full_list, full_list ), container.getSamplesCount() );
			IntermediateHybridTreeNode negative_tree = IntermediateHybridTreeNode( mNegativeMap, new PopTreePreliminaryCondition( mNegativeMap, minCommonBits, full_list, full_list ), container.getSamplesCount() );

            BitSelector positive_selector( MostCommonBitFirstStrategy ( mPositiveEntries, mPlane ));
            BitSelector negative_selector( MostCommonBitFirstStrategy ( mNegativeEntries, mPlane ));

            HybridTreeCreatorUtils positive_utils( mPositiveEntries.size()/4096 + 8, mCutMap, positive_selector, mPlane, container );
            HybridTreeCreatorUtils negative_utils( mNegativeEntries.size()/4096 + 8, mCutMap, negative_selector, mPlane, container );

			positive_tree.partition( mPositiveEntries, positive_utils, 0 );
			negative_tree.partition( mNegativeEntries, negative_utils, 0 );

            positive_tree.optimize();
            negative_tree.optimize();

			std::vector<const void*> candidates;

	        START_TIMER( PRELIMINARY_PHASE );

			positive_tree.preliminaryTraverse( &negative_tree, candidates );

	        STOP_TIMER( PRELIMINARY_PHASE );
            stats.eliminationTime = TIMER_VALUE( PRELIMINARY_PHASE );
            stats.verificationCandidates = candidates.size();

            std::vector<bool> best;

            do {
                SamplesContainer ver_container = ver_creator.createSamples( mZeroEntries, mPlane, 1 );

                IntermediateHybridTreeNode zero_tree = IntermediateHybridTreeNode( mZeroMap, new PopTreePreliminaryCondition( mZeroMap, minCommonBits, full_list, full_list ), ver_container.getSamplesCount() ); 
                BitSelector zero_selector( MostCommonBitFirstStrategy ( mZeroEntries, mPlane ));
                HybridTreeCreatorUtils zero_utils( mZeroEntries.size()/4096 + 8, mCutMap, zero_selector, mPlane, ver_container );

                zero_tree.partition( mZeroEntries, zero_utils, 0 );
                zero_tree.optimize();

#ifdef __COMBINATIONAL_TEST__
                START_TIMER( VERIFICATION_PHASE );

    #ifdef __STATS_OUTPUT__
                VerificationCondition::start_new_stat_session();
                LeafNode::start_new_stat_session();
    #endif
#endif
                buffer.clear();

                findAdjacentRays( candidates, zero_tree, ver_container, buffer );

#ifdef __COMBINATIONAL_TEST__
                STOP_TIMER( VERIFICATION_PHASE );
    
                if ( LeafNode::LEAF_NODE_VISITS < stats.leafNodeVisits || stats.leafNodeVisits == 0 ) {

                    best = ver_creator.current();

                    stats.verificationTime += TIMER_VALUE( VERIFICATION_PHASE );

    #ifdef __STATS_OUTPUT__
                    stats.elimOps = PreliminaryCondition::OPS;
                    stats.verOps = VerificationCondition::OPS;
                    stats.leafNodeVisits = LeafNode::LEAF_NODE_VISITS;
                    stats.totalLeafNodes = zero_tree.getLeafNodesCount();
                    stats.ineffectiveLeafNodeVisits = LeafNode::INEFFECTIVE_LEAF_NODE_VISITS;
                    stats.averageMapPopulation = LeafNode::AVERAGE_MAP_POPULATION;
                    stats.averageRaysPerLeaf = LeafNode::AVERAGE_RAYS_PER_LEAF;
    #endif
                }
#endif
            } while ( ver_creator.hasMorePermutations() );

            DD_OUTPUT( std::cout << "That's the best we found:" )
            DD_OUTPUT( "################################################################")
            DD_PRINT_VECTOR_ELEMENTARY( best )
            DD_OUTPUT( "################################################################")

			return stats;
		}
#endif

		void clear() {
			//@ToDo Clean up here!
		}
	};

template <class RAY_TYPE>
	const unsigned short PopulationTreeCacheEntry<RAY_TYPE>::PATTERN_SET_A_SIZE = 10;

template <class RAY_TYPE>
	const unsigned short PopulationTreeCacheEntry<RAY_TYPE>::PATTERN_SET_B_SIZE = 12;

template <class RAY_TYPE>
	const unsigned short PopulationTreeCacheEntry<RAY_TYPE>::PATTERN_SET_C_SIZE = 16;

template <class RAY_TYPE>
	const unsigned short PopulationTreeCacheEntry<RAY_TYPE>::PATTERN_SET_D_SIZE = 16;

template <class RAY_TYPE>
	const unsigned short PopulationTreeCacheEntry<RAY_TYPE>::PATTERN_SET_E_SIZE = 1;

#if defined _LP64 || defined __LP64__

	template <class RAY_TYPE>
	const unsigned long PopulationTreeCacheEntry<RAY_TYPE>::PATTERN_SET_A[] = {
		0x5555555555555555,
		0x9999999999999999,
		0xF0F0F0F0F0F0F0F0,
		0x3333333333333333, 
		0xC3C3C3C3C3C3C3C3,
		0x6969696969696969,
		0xA5A5A5A5A5A5A5A5,
		0x0000FFFF0000FFFF,
		0x00FF00FF00FF00FF,
		0xD2D2D2D2D2D2D2D2
	};

	template <class RAY_TYPE>
	const unsigned long PopulationTreeCacheEntry<RAY_TYPE>::PATTERN_SET_B[] = {
		0x5555555555555555,
		0x9999999999999999,
		0xF0F0F0F0F0F0F0F0,
		0x3333333333333333, 
		0xC3C3C3C3C3C3C3C3,
		0x6969696969696969,
		0xA5A5A5A5A5A5A5A5,
		0x0000FFFF0000FFFF,
		0x00FF00FF00FF00FF,
		0xD2D2D2D2D2D2D2D2,
		0xE5E5E5E5E5E5E5E5,
		0xC9C9C9C9C9C9C9C9
	};

	template <class RAY_TYPE>
	const unsigned long PopulationTreeCacheEntry<RAY_TYPE>::PATTERN_SET_C[] = {
		0x0F1E2D3C4B5A6978,
		0x102F3E4D5C6B7A89,
		0x21304F5E6D7C8B9A,
		0x3241506F7E8D9CAB,
		0x435261708F9EADBC,
		0x5463728190AFBECD,
		0x65748392A1B0CFDE,
		0x768594A3B2C1D0EF,
		0x8796A5B4C3D2E1F0,
		0x98A7B6C5D4E3F201,
		0xA9B8C7D6E5F40312,
		0xBAC9D8E7F6051423,
		0xCBDAE9F807162534,
		0xDCEBFA0918273645,
		0xEDFC0B1A29384756,
		0xFE0D1C2B3A495867
	};

	template <class RAY_TYPE>
	const unsigned long PopulationTreeCacheEntry<RAY_TYPE>::PATTERN_SET_D[] = {
		0x0F1E2D3C4B5A6978,
		0x21304F5E6D7C8B9A,
		0x435261708F9EADBC,
		0x65748392A1B0CFDE,
		0x8796A5B4C3D2E1F0,
		0x102F3E4D5C6B7A89,
		0x5463728190AFBECD,
		0x98A7B6C5D4E3F201,
		0xBAC9D8E7F6051423,
		0xDCEBFA0918273645,
		0xFE0D1C2B3A495867,
		0x3241506F7E8D9CAB,
		0x768594A3B2C1D0EF,
		0xA9B8C7D6E5F40312,
		0xCBDAE9F807162534,
		0xEDFC0B1A29384756
	};

	template <class RAY_TYPE>
	const unsigned long PopulationTreeCacheEntry<RAY_TYPE>::PATTERN_SET_E[] = {
		0x0F1E2D3C4B5A6978,
		0xFFFFFFFFFFFFFFFF,
		0xFFFFFFFFFFFFFFFF,
		0xFFFFFFFFFFFFFFFF
	};

#else
	template <class RAY_TYPE>
	const unsigned long PopulationTreeCacheEntry<RAY_TYPE>::PATTERN_SET_A[] = {
		0x55555555,
		0x99999999,
		0xF0F0F0F0,
		0x33333333, 
		0xC3C3C3C3,
		0x69696969,
		0xA5A5A5A5,
		0x0000FFFF,
		0x00FF00FF,
		0xD2D2D2D2
	};

	template <class RAY_TYPE>
	const unsigned long PopulationTreeCacheEntry<RAY_TYPE>::PATTERN_SET_B[] = {
		0x55555555,
		0x99999999,
		0xF0F0F0F0,
		0x33333333, 
		0xC3C3C3C3,
		0x69696969,
		0xA5A5A5A5,
		0x0000FFFF,
		0x00FF00FF,
		0xD2D2D2D2,
		0xE5E5E5E5,
		0xC9C9C9C9
	};
#endif // _LP64 || defined __LP64__
}

#endif // __DD_POPULATION_TREE_CACHE_ENTRY__
