/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_PRELIMINARY_TRAVERSE_VISITOR__
#define __DD_PRELIMINARY_TRAVERSE_VISITOR__

#include <deque>
#include <vector>

#include "DDTreeNode.h"
#include "DDRayRefVisitor.h"
#include "DDRayCache.h"
#include "DDBitPatternTreeCacheEntry.h"

namespace double_description {

    template <class RAY_TYPE>
    class PreliminaryTraverseVisitor : public RayRefVisitor {

        protected:
        const unsigned short mPlane;
        const unsigned short mRows;
        RayCache<RAY_TYPE, BitPatternTreeCacheEntry<RAY_TYPE> > & mCache;

    #ifdef __STATS_OUTPUT__
        unsigned long mVerified;
    #endif

        const unsigned short mMinCommonBits;
        const TreeNode & mTree;
        std::deque<struct EdgeCandidate> mCandidates;

        bool hasFinished () const {
            return true;
        }

        public:
        PreliminaryTraverseVisitor( const TreeNode & tree,
                                    const unsigned short bits,
                                    const unsigned short plane,
                                    const unsigned short rows,
                                    RayCache<RAY_TYPE, BitPatternTreeCacheEntry<RAY_TYPE> > & cache ) :
            RayRefVisitor(),
            mPlane( plane ),
            mRows( rows ),
            mCache( cache ),
    #ifdef __STATS_OUTPUT__
            mVerified( 0 ),
    #endif
            mMinCommonBits( bits ),
            mTree( tree ) {}

        void onStart () {
            RayRefVisitor::onStart();

            mCandidates.clear();
        }

    #ifdef __STATS_OUTPUT__
        unsigned long verified () const  {
            return mVerified;
        }
    #endif

        virtual ~PreliminaryTraverseVisitor () {}
    };
}

#endif // __DD_PRELIMINARY_TRAVERSE_VISITOR__
