/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_RANK_CALCULATOR__
#define __DD_RANK_CALCULATOR__

#include "DDBitList.h"
#include "DDCalculationMatrix.h"

namespace double_description {

	class RankCalculator : protected CalculationMatrix {

		public:
		RankCalculator ( const unsigned long rows, const unsigned long columns );
		virtual ~RankCalculator() {}

		void load ( const Matrix& matrix, const BitList & rowMap ) {
            unsigned int last_row = 0;

            for ( unsigned short block = 0; block < rowMap.getBlocks(); block++ ) {
                unsigned long block_data = rowMap[ block ];

			    while ( block_data ) {
				    const unsigned long suffix = __builtin_ctzl( block_data ); 
				    const unsigned long mask = __SINGLE_BIT__ << suffix;

                    const CalculationVector & matrix_vec = matrix.getRow( block * __BLOCK_SIZE__ + suffix );
                    CalculationVector & calculator_vec = getRow( last_row++ );

					calculator_vec = matrix_vec;

				    block_data ^= mask;
			    }
            }


/*
			for ( unsigned long row = 0; row < matrix.getRows(); row++ ) {
				if ( rowMap.isSet( row ) ) {

					for ( unsigned long column = 0; column < matrix.getColumns(); column++ ) {
						setValue( last_row, column, matrix[row][column] );
					}
					last_row++;
				}
			}
*/
			setRows( last_row );
			setColumns( matrix.getColumns() );
		}

        void addRow ( const CalculationVector & vec ) {

            unsigned long last_row = getRows();
            setRows( last_row + 1 );

		    for ( unsigned long column = 0; column < vec.getDimension(); column++ ) {
			    setValue( last_row, column, vec[column] );
			}
        }

        void removeRow () {
            setRows( getRows() - 1 );
        }

        void overwriteLastRow ( const CalculationVector & vec ) {

		    for ( unsigned long column = 0; column < vec.getDimension(); column++ ) {
			    setValue( getRows() - 1, column, vec[column] );
			}
        }

		bool hasMinimumRank ( unsigned long rank ) {
			unsigned long row, column;
			long clearance = getColumns() - rank;

			for ( row = 0, column = 0; row < getRows() && column < getColumns(); column++ ) {
				if ( findPivot( row, column ) ) {
                    eliminateColumnDown( row, getRows(), column );
					row++;
				} else if ( --clearance < 0 ) {
					return false;
				}
			}

			return true;
		}

		bool hasMinimumRank ( const Matrix& matrix, const BitList& rowMap, const unsigned long rank ) {
			load( matrix, rowMap );
			return hasMinimumRank( rank );
		}
	};
}

#endif // __DD_RANK_CALCULATOR__
