/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_RAY_REF_VISITOR__
#define __DD_RAY_REF_VISITOR__

#include "DDRayReference.h"

namespace double_description {

    enum VisitorState {
        VS_IDLE,
        VS_INIT,
        VS_RUNNING,
        VS_WAITING
    };

    class RayRefVisitor {

        protected:
        VisitorState mState;

        virtual bool hasFinished () const = 0;
    
        public:
        RayRefVisitor () :
            mState( VS_IDLE ) {}

        virtual ~RayRefVisitor () {}

        virtual void onStart () {
            mState = VS_INIT;
        }

        virtual void onVisit ( const struct RayReference & ) {
            mState = VS_RUNNING;
        }

        virtual void onFinish () {
            if ( hasFinished() ) {
                mState = VS_IDLE;
            } else {
                mState = VS_WAITING;
            }
        }

        VisitorState getState () const {
            return mState;
        }
    };
}

#endif // __DD_RAY_REF_VISITOR__
