
#ifndef __DD_SAMPLE_CREATOR__
#define __DD_SAMPLE_CREATOR__

#include <vector>

#include "DDSamplesContainer.h"

namespace double_description {

    struct PopulationEntry {
        unsigned short position;
        unsigned long count;

        PopulationEntry( const unsigned short pos ) :
            position( pos ),
            count( 0 ) {}

        bool operator< ( const struct PopulationEntry & rhs ) const {
            return count > rhs.count;
        }
    };

    class SampleCreator {

        protected:
        const unsigned short mBlocks;
        const unsigned short mRegularThreshold;
        const unsigned short mInverseThreshold;

        const std::vector<struct PopulationEntry> createStats ( const std::vector<struct RayReference>& entries,
                                                                const unsigned int plane ) const {
            std::vector<struct PopulationEntry> population;

            for ( unsigned short pos = 0; pos < plane; pos++ ) {
                population.push_back( PopulationEntry( pos ) );
            }

			for ( unsigned long index = 0; index < entries.size(); index++ ) {
				for ( unsigned short pos = 0; pos < plane; pos++ ) {
					if ( entries[index].activeSet->isSet( pos ) ) {
						population[ pos ].count++;
					}
				}
			}

            return population;
        }

        const BitList getInverseBasicPattern ( const BitList & basicPatternReg,
                                               const unsigned short plane ) const {

            BitList result = ~basicPatternReg;
            result.unsetBitsFrom( plane ); 

            return result;
        }

        unsigned long getOverCount ( const SamplesContainer & container,
                                     const std::vector<struct RayReference>& entries ) const {
            unsigned long result = 0;            

            for ( unsigned long i = 0; i < entries.size(); i++ ) {
                if ( container.isRegularOver( *(entries[i].activeSet) ) &&
                     !container.isInverseOver( *(entries[i].activeSet) ) ) {
                    result++;
                }
            }

            return result;
        }

        unsigned long getDoubleOverCount ( const SamplesContainer & container,
                                           const std::vector<struct RayReference>& entries ) const {
            unsigned long result = 0;            

            for ( unsigned long i = 0; i < entries.size(); i++ ) {
                if ( container.isRegularOver( *(entries[i].activeSet) ) &&
                     container.isInverseOver( *(entries[i].activeSet) ) ) {
                    result++;
                }
            }

            return result;
        }

        public:
        SampleCreator ( const unsigned short blocks,
                        const unsigned short minCommonBits ) :
            mBlocks( blocks ),
            mRegularThreshold( minCommonBits % 2 == 0 ? minCommonBits/2 + 1 : (minCommonBits + 1)/2 ),
            mInverseThreshold( minCommonBits % 2 == 0 ? minCommonBits/2 : (minCommonBits + 1)/2 ) {}            

        virtual const SamplesContainer createSamples( const std::vector<struct RayReference>& entries,
                                                      const unsigned int plane,
                                                      const unsigned int maxSamples ) const = 0; 

        virtual const SamplesContainer createSamples( const std::vector<struct RayReference>& posEntries,
                                                      const std::vector<struct RayReference>& negEntries,
                                                      const unsigned int plane,
                                                      const unsigned int maxSamples ) const = 0; 
    };
}

#endif // __DD_SAMPLE_CREATOR__
