/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_SINGLE_BIT_SPLIT_VISITOR__
#define __DD_SINGLE_BIT_SPLIT_VISITOR__

#include "DDBitSplitVisitor.h"
#include "DDBitSelectStrategy.h"
#include "DDBitSelector.h"

namespace double_description {

    enum SingleBitSplitVisitorSets {
        SINGLE_BIT_SPLIT_SUB_SET     = 0,
        SINGLE_BIT_SPLIT_SUPER_SET   = 1
    };

    class SingleBitSplitVisitor : public BitSplitVisitor {

        private:
        BitSelector mSelector;
        unsigned short mSplitBit;

        public:
        SingleBitSplitVisitor( const unsigned short depth,
                               const unsigned short capacity,
                               const BitSelectStrategy & strategy ) :
            BitSplitVisitor( depth, capacity, 2 ),
            mSelector( strategy ),
            mSplitBit( 0 ) {}

        void onVisit ( const struct RayReference & ref ) {

		    if ( ref.activeSet.isSet( mSplitBit ) ) {
			    set( SINGLE_BIT_SPLIT_SUPER_SET ).insert( ref );
		    } else {
			    set( SINGLE_BIT_SPLIT_SUB_SET ).insert( ref );
		    }
        }

        void onFinish () {}    

        void before ( const RayReferenceContainer & current ) {

            BitSplitVisitor::before( current );

            mSplitBit = mSelector.getNextBit( current.union_map() - current.cut_map() );
        }

        void after () {
            BitSplitVisitor::after();
            mSelector.revert();
        }

        bool hasFinished () const {
            return true;
        } 

        bool can_proceed ( const unsigned short index ) const {
            return BitSplitVisitor::can_proceed( index ) && mSelector.checkNextBit( set( index ).union_map() - set( index ).cut_map() );
        }
    };
}

#endif // __DD_SINGLE_BIT_SPLIT_VISITOR__
