/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_SINGLE_SET_RAY__
#define __DD_SINGLE_SET_RAY__

#include "DDBitList.h"
#include "DDRay.h"

#include <string.h>

#include <gmp.h>
#include <gmpxx.h>

namespace double_description {

	class SingleSetRay : public Ray {

		private:
		BitList mActiveSet;

		public:
		SingleSetRay ( const unsigned long dimension, mpz_class* const coeffs, const BitList & initialActiveSet ) :
			Ray( dimension, coeffs ),
			mActiveSet( initialActiveSet ) {}

		SingleSetRay ( const RepresentationVector & vector, mpz_class* const coeffs, const BitList & initialActiveSet ) :
			Ray( vector, vector.getDimension() - 1, coeffs ),
			mActiveSet( initialActiveSet ) {}

		SingleSetRay ( const SingleSetRay* const positive, const SingleSetRay* const negative, const unsigned long plane, const unsigned long totalPlanes ) :
			Ray( positive, negative, plane, totalPlanes ),
			mActiveSet( positive->getActiveSet() ) {
				mActiveSet &= negative->getActiveSet();
				mActiveSet.setBit( plane );

				for ( unsigned long next_plane = plane + 1; next_plane < totalPlanes && getCachedPlaneCoeff( next_plane ) >= 0; next_plane++ ) {
					if ( getCachedPlaneCoeff( next_plane ) == 0 ) {
						mActiveSet.setBit( next_plane );
					}
				}
		}

		virtual ~SingleSetRay () {}

		const BitList& getActiveSet () const {
			return mActiveSet;
		}

		const BitList& getMinimalActiveSet () const {
			return mActiveSet;
		}

        bool degenerate () const {
            return mActiveSet.getPopulationCount() == getDimension() - 1;
        }

		void copyActiveSetData ( unsigned long * buffer, const unsigned short blocks ) const {

			buffer[0] = mActiveSet[0];

			for ( unsigned long block = 1; block < blocks; block++ ) {
				buffer[block] = mActiveSet[block];
			}
		}

		void copyActiveSetData ( unsigned long * buffer ) const {

			for ( unsigned long block = 0; block < mActiveSet.getBlocks(); block++ ) {
				buffer[block] = mActiveSet[block];
			}
		}

		void copyActiveSetData ( unsigned long * buffer, const unsigned long * const mask ) const {

			for ( unsigned long block = 0; block < mActiveSet.getBlocks(); block++ ) {
				buffer[block] = mActiveSet[block] & mask[block];
			}
		}

		void setActiveBit ( const unsigned long position ) {
			mActiveSet.setBit( position );
		}

		const std::string toString () const;
	};
}

#endif // __DD_SINGLE_SET_RAY__
