/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_SINGLE_SET_RAY_ENUMERATOR__
#define __DD_SINGLE_SET_RAY_ENUMERATOR__

#include "DDExtremeRayEnumerator.h"

#include "DDCombTestTraverseVisitor.h"
#include "DDAlgTestTraverseVisitor.h"

namespace double_description {

    class SingleSetRayEnumerator : public ExtremeRayEnumerator<SingleSetRay> {

        protected:
        unsigned long enumerate_pos_deg ( const TreeNode * const pos_deg_tree ) {

            if ( Options::current().adjacency_test() == COMBINATIONAL_TEST ) {
                const TreeNode * verificator = createVerTree();
                CombTestTraverseVisitor verification_visitor( *pos_deg_tree, mMinCommonBits, mPlane, mSystem.getRows(), *verificator, mCache );
                mNegativeEntries.iterate( verification_visitor );
                delete verificator;

                return verification_visitor.verified();
            } else {
                AlgTestTraverseVisitor verification_visitor( *pos_deg_tree, mPlane, mSystem, mCache );
                mNegativeEntries.iterate( verification_visitor );

                return verification_visitor.verified();
            }
        }

        unsigned long enumerate_pos_nondeg ( const TreeNode * const pos_nondeg_tree ) {
            SingleSetTraverseVisitor narrowing_visitor( *pos_nondeg_tree, mMinCommonBits, mPlane, mSystem.getRows(), mCache );
            mNegativeEntries.iterate( narrowing_visitor );

            return narrowing_visitor.verified();
        }

        public:
        SingleSetRayEnumerator ( const Matrix & system,
                                 const unsigned short plane,
                                 const BitPatternTreeCacheEntry<SingleSetRay> & entry,
                                 RayCache<SingleSetRay, BitPatternTreeCacheEntry<SingleSetRay> > & cache ) :
            ExtremeRayEnumerator( system, plane, entry, cache ) {}
    };
}

#endif // __DD_SINGLE_SET_RAY_ENUMERATOR__
