/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <sys/time.h>

#include "DDSingleSetRayGenerator.h"
#include "DDSingleSetRayEnumerator.h"

using namespace double_description;

void SingleSetRayGenerator::calculateBase () {

    ExtremeRayUncompressor decompressor( mSystem.getRows(), mSystem.getColumns(), BitList::getBlocks( mSystem.getRows() ));

	for ( unsigned long row = 0; row < mSystem.getColumns(); row++ ) {

		BitList surface_map( BitList::getBlocks( mSystem.getRows() ) );
		RepresentationVector vector( mSystem.getColumns() );
		SingleSetRay* ray;
		mpz_class* coeffs = new mpz_class[ mSystem.getRows() - mSystem.getColumns() ];

        surface_map.setBits( 0, mSystem.getColumns() );
		surface_map.unsetBit( row );

        decompressor.calcExtremeRay( mSystem, surface_map, vector );

		if ( vector * mSystem[row] < 0 ) {
			vector.negate();
		}

		unsigned long index;

		for ( index = mSystem.getColumns(); index < mSystem.getRows(); index++ ) {
			vector.calcScalarProduct( mSystem[index], coeffs[ index - mSystem.getColumns() ] );
			if ( coeffs[ index - mSystem.getColumns() ] == 0 ) {
				surface_map.setBit( index );
			} else if ( coeffs[ index - mSystem.getColumns() ] < 0 ) {
				break;
			}
		}

		for ( ; index < mSystem.getRows(); index++ ) {
			vector.calcScalarProduct( mSystem[index], coeffs[ index - mSystem.getColumns() ] );
		}
		
		ray = new SingleSetRay( vector, coeffs, surface_map );

		mCache.insert( ray );
	}
}

const struct StepStats SingleSetRayGenerator::enumerate ( const unsigned long plane, BitPatternTreeCacheEntry<SingleSetRay> & cache_entry ) {
    SingleSetRayEnumerator enumerator( mSystem, plane, cache_entry, mCache );
    
    return enumerator.enumerate();
}
