/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_SINGLE_SET_TRAVERSE_VISITOR__
#define __DD_SINGLE_SET_TRAVERSE_VISITOR__

#include "DDPreliminaryTraverseVisitor.h"
#include "DDSingleSetRay.h"

namespace double_description {

    class SingleSetTraverseVisitor : public PreliminaryTraverseVisitor<SingleSetRay> {


        protected:

        void create ( const EdgeCandidate & verified ) {
	        const SingleSetRay * positive = static_cast<const SingleSetRay*>( verified.first.ray );
			const SingleSetRay * negative = static_cast<const SingleSetRay*>( verified.second.ray );               

			mCache.insert( new SingleSetRay( positive, negative, mPlane, mRows ), 1 );                

    #ifdef __STATS_OUTPUT__
            mVerified++;
    #endif
        }

        public:
        SingleSetTraverseVisitor( const TreeNode & tree,
                                  const unsigned short bits,
                                  const unsigned short plane,
                                  const unsigned short rows,
                                  RayCache<SingleSetRay, BitPatternTreeCacheEntry<SingleSetRay> > & cache ) :
            PreliminaryTraverseVisitor( tree, bits, plane, rows, cache ) {}

        virtual void onVisit ( const struct RayReference & current ) {
            PreliminaryTraverseVisitor::onVisit( current );

            mTree.preliminaryTraverse( current, mCandidates );

            for ( std::deque<struct EdgeCandidate>::const_iterator it = mCandidates.begin(); it != mCandidates.end(); it++ ) {
                create( *it );
            }

            mCandidates.clear();
        } 
    };
}

#endif // __DD_SINGLE_SET_TRAVERSE_VISITOR__
