
#ifndef __DD_STATIC_SAMPLE_CREATOR__
#define __DD_STATIC_SAMPLE_CREATOR__

#include "DDSampleCreator.h"

namespace double_description {

    class StaticSampleCreator : public SampleCreator {

        private:
        const float mLimit;

        protected:
        virtual const std::vector<unsigned long> getBasicSamples () const = 0;

        public:
        StaticSampleCreator ( const unsigned short blocks,
                              const unsigned short minCommonBits,
                              const float limit ) :
            SampleCreator( blocks, minCommonBits ),
            mLimit( limit ) {}

        const SamplesContainer createSamples( const std::vector<struct RayReference>& entries,
                                              const unsigned int plane,
                                              const unsigned int maxSamples ) const {

            SamplesContainer container( maxSamples, mRegularThreshold, mInverseThreshold );
            const std::vector<unsigned long> basic_samples = getBasicSamples();       

            for ( unsigned int i = 0; i < basic_samples.size(); i++ ) {
                const BitList sample( mBlocks, basic_samples[i] );

                container.insert( sample, getInverseBasicPattern( sample, plane ) );
                if ( getDoubleOverCount( container, entries ) >= ( 1.0 - mLimit ) * entries.size() ) {
                    container.remove();
                }
            }

            return container;
        }

        const SamplesContainer createSamples( const std::vector<struct RayReference> &,
                                                      const std::vector<struct RayReference> &,
                                                      const unsigned int,
                                                      const unsigned int maxSamples ) const {

            SamplesContainer container( maxSamples, mRegularThreshold, mInverseThreshold );

            return container;
        }
    };
}

#endif // __DD_STATIC_SAMPLE_CREATOR__
