/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_STATS__
#define __DD_STATS__

#define START_TIMER(ID) struct timeval start_##ID, end_##ID; \
	                    long mtime_##ID, seconds_##ID, useconds_##ID; \
                        gettimeofday( &start_##ID, NULL );

#define STOP_TIMER(ID)  gettimeofday( &end_##ID, NULL ); \
                        seconds_##ID  = end_##ID.tv_sec  - start_##ID.tv_sec; \
	                    useconds_##ID = end_##ID.tv_usec - start_##ID.tv_usec; \
                        mtime_##ID = ((seconds_##ID) * 1000 + useconds_##ID/1000.0) + 0.5;

#define TIMER_VALUE(ID) mtime_##ID

namespace double_description {

    struct StepStats {
        unsigned long verOps;
        unsigned long elimOps;
        unsigned long eliminationTime;
        unsigned long verificationTime;
        unsigned long verificationCandidates;
        unsigned long verified;
        unsigned long leafNodeVisits;
        unsigned long totalLeafNodes;
        unsigned long ineffectiveLeafNodeVisits;
        double averageMapPopulation;
        double averageRaysPerLeaf;
        double degeneracyPos;
        double degeneracyNeg;
        double degeneracyZero;
        double elimComplexity;
        double verComplexityLg;
        double verComplexityLin;

        StepStats () :
            verOps( 0 ),
            elimOps( 0 ),
            eliminationTime( 0 ),
            verificationTime( 0 ),
            verificationCandidates( 0 ),
            verified( 0 ),
            leafNodeVisits( 0 ),
            totalLeafNodes( 0 ),
            ineffectiveLeafNodeVisits( 0 ),
            averageMapPopulation( 0.0 ),
            averageRaysPerLeaf( 0.0 ),
            degeneracyPos( 0.0 ),
            degeneracyNeg( 0.0 ),
            degeneracyZero( 0.0 ),
            elimComplexity( 0.0 ),
            verComplexityLg( 0.0 ),
            verComplexityLin( 0.0 ) {}
    };
}

#endif // __DD_STATS__
