/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_ROW_VECTOR__
#define __DD_ROW_VECTOR__

#include <vector>
#include <string.h>

#include <gmp.h>
#include <gmpxx.h>

#include "DDUtilsFactory.h"

namespace double_description {

	class Vector {

		protected:
		std::vector<mpz_class> data;

		Vector () {}

		Vector ( const unsigned int dataSize ) :
			data( dataSize, 0 ) {}

		Vector ( const unsigned int dataSize, const mpz_class & value ) :
			data( dataSize, value ) {}

        Vector ( const Vector & other ) :
            data( other.data ) {}

        void clear () {
            data.clear();
        }

		public:
		virtual ~Vector () {}

		void simplify () {
	        unsigned int index;

	        UtilsFactory::CALC_UTILS.startIncrementalGCD( data[0] );

	        for ( index = 1; index < data.size(); index++ ) {
		        UtilsFactory::CALC_UTILS.calcIncrementalGCD( data[index] );
	        }

	        UtilsFactory::CALC_UTILS.convertResultToAbsoluteValue();

	        for ( index = 0; index < data.size(); index++ ) {
		        data[index] /= UtilsFactory::CALC_UTILS.getResult();
	        }	
        }

		void operator *= ( const long scalar ) {
			for ( unsigned int index = 0; index < data.size(); index++ ) {
				data[index] *= scalar;
			}
		}

		void operator *= ( const mpz_class& scalar ) {
			for ( unsigned int index = 0; index < data.size(); index++ ) {
				data[index] *= scalar;
			}
		}

		void operator /= ( const mpz_class & scalar ) {
			for ( unsigned int index = 0; index < data.size(); index++ ) {
				data[index] /= scalar;
			}
		}

		void operator = ( const Vector& vector ) {
			data = vector.data;
		} 
	};
}

#endif // __DD_ROW_VECTOR__
