/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <stdlib.h>
#include <sys/time.h>

#include "DDInput.h"
#include "DDOutput.h"
#include "DDStats.h"

#include "DDOutputVector.h"
#include "DDRepresentationVector.h"

#include "DDLogging.h"

#include "DDOptions.h"

using namespace double_description;

template <class GENERATOR>
int generate ( const char * problem ) {

	struct timeval start, end;
	long mtime, seconds, useconds;

	Input input ( problem );
    const Options & options = Options::current();

    DD_OUTPUT( options.toString() )

    DD_OUTPUT ( "-----------------------------------------------------------------------------------" )
    DD_OUTPUT ( "|                                  COMPUTATION                                    |" )
    DD_OUTPUT ( "-----------------------------------------------------------------------------------" )

#ifdef __COMPRESSION__
	GENERATOR generator( input.getSystem(), input.getTransformedSystem(), input.getTransformator() );
#else
	GENERATOR generator( input.getTransformedSystem(), input.getTransformator() );
#endif

	gettimeofday(&start, NULL);	


#ifdef __STATS_OUTPUT__
    if ( options.plot_required() ) {
        OPEN_LOG( COMPLEXITY_PLOT )
        OPEN_LOG( VER_BIT_DISTRIBUTION_PLOT )
    }
#endif

    generator.calculateBase();

    START_TIMER( ITERATIONS_TIME )
    generator.run();
    STOP_TIMER( ITERATIONS_TIME )

#ifdef __STATS_OUTPUT__
    if ( options.plot_required() ) {
        DUMP_LOG( COMPLEXITY_PLOT, options.plot_file().c_str() )
        DUMP_LOG( VER_BIT_DISTRIBUTION_PLOT, "ver_bit_distribution.plt" )
    }
#endif

    const std::string res_file = std::string( problem ) + ".out";

    if ( options.output_format() == INTEGER_VEC ) {
    	generator.template writeExtremeRays<RepresentationVector>( res_file.c_str() );
    } else if ( options.output_format() == DOUBLE_VEC ) {
        generator.template writeExtremeRays<OutputVector<double> >( res_file.c_str() );
    } else {
        generator.template writeExtremeRays<BitList>( res_file.c_str() );
    }

	gettimeofday(&end, NULL);

	seconds  = end.tv_sec  - start.tv_sec;
	useconds = end.tv_usec - start.tv_usec;

	mtime = ((seconds) * 1000 + useconds/1000.0) + 0.5;

    DD_OUTPUT( "Narrowing and verification time: " << TIMER_VALUE( ITERATIONS_TIME ) << "ms" )
	DD_OUTPUT( "Overall time: " << mtime << "ms" );

    Options::release();

	return 0;
}
