/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "DDInput.h"
#include "DDOutput.h"
#include "DDLogging.h"

using namespace double_description;

int main ( int argc, const char** argv ) {

	if ( argc != 3 ) {

        DD_OUTPUT("***********************************************************************************" )
        DD_OUTPUT("* Addibit: Another Double Description Implementation with BInary Trees.           *" )
        DD_OUTPUT("* Copyright (C) 2013  Blagoy Genov                                                *" )
        DD_OUTPUT("* This program comes with ABSOLUTELY NO WARRANTY!!!                               *" )
        DD_OUTPUT("***********************************************************************************\n" )

		DD_OUTPUT( "Usage: addibit-export porta|4ti2 MATRIX_FILE" );
		return -1;
	}

    Input input( argv[2] );
    const Matrix & system = input.getSystem();
    const std::string prog = argv[1];
    const std::string name = std::string( argv[2] ).substr( 0, std::string( argv[2] ).find( ".ine" ) );

    if ( prog == "porta" ) {
        const std::string res_file = name + ".ieq";
        Output output( res_file.c_str() );
        output.write( system.toPortaFormat() );
    } else if ( prog == "4ti2" ) {
        const std::string res_mat_file = name + ".mat";
        const std::string res_rel_file = name + ".rel";
        const std::string res_sign_file = name + ".sign";

        Output mat_output( res_mat_file.c_str() );
        mat_output.write( system.to4ti2Format() );        

        Output rel_output( res_rel_file.c_str() );
        rel_output.write( system.to4ti2RelFormat() );  

        Output sign_output( res_sign_file.c_str() );
        sign_output.write( system.to4ti2SignFormat() ); 
    } else {
        DD_OUTPUT( prog << " is not a valid option. Use porta or 4ti2." )
    }
}
